/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
*** Copyright (c) 2000-2002 Regents of the University of Michigan,
***               Saurabh N. Adya and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/








#include "ABKCommon/abkcommon.h"
#include "RBPlace/RBPlacement.h"
#include "CongestionMaps.h"

int main(int argc, const char *argv[])
{
  NoParams         noParams       (argc,argv);
  BoolParam        helpRequest    ("help",argc,argv);
  BoolParam        helpRequest1   ("h",   argc,argv);
  StringParam      plotCongMap("plotCongMap", argc, argv);
  StringParam      plotCongMapGP("plotCongMapGP", argc, argv);
  StringParam      plotCongMapXPM("plotCongMapXPM", argc, argv);
  BoolParam        useProbability   ("useProb",   argc,argv);

  RBPlacement::Parameters rbParams(argc,argv);
  if (noParams.found() || helpRequest.found() || helpRequest1.found())
  {
     cout << "  Use '-help' or '-f filename.aux' " << endl;
     cout << "  Other options: \n"
          << "  -useProb (use Probabilistic congestion map)\n"
          << "  -plotCongMap base_filename \n"
          << "  -plotCongMapGP base_filename \n"
          << "  -plotCongMapXPM base_filename \n"
          << endl;
     return 0;
  }
  StringParam auxFile("f", argc, argv);
  abkfatal(auxFile.found(), "must have -f <auxfilename>");
  RBPlacement rbplace(auxFile, rbParams);

  CongestionMaps congMap(&rbplace);
  if(!useProbability.found())
    {
      /*
      congMap.initializeFMatrix();
      vector<unsigned> cellIds;
      for(unsigned i=0; i<rbplace.getNumCells(); ++i)
	cellIds.push_back(i);
      congMap.addCongByCells(cellIds, 0);
      */
      congMap.createCongestionMap();
    }
  else
    {
      congMap.initializeFMatrix();
      congMap.createCongestionMapProb();
      /*
      vector< vector<double> > Px;
      vector< vector<double> > Py;
      Point ptLeft(100, 100);
      Point ptRight(20000, 20000);
      congMap.computePMatrix(Px, Py, ptLeft, ptRight);
      for(int i=Px.size()-1; i>=0; --i)
	{
	  for(unsigned j=0; j < Px[i].size(); ++j)
	    cout<<setw(10)<<Px[i][j]<<" ";
	  cout<<endl;
	}
      cout<<endl;
      for(int i=Px.size()-1; i>=0; --i)
	{
	  for(unsigned j=0; j < Px[i].size(); ++j)
	    cout<<setw(10)<<Py[i][j]<<" ";
	  cout<<endl;
	}
      */
    }

  congMap.sortCongestionBins();
  cout<<"Peak Congestion is "
      <<congMap.sortCongBins[congMap.sortCongBins.size()-1].totalCongestion
      <<endl;
  if(plotCongMap.found())
    congMap.plotMatCongMap(plotCongMap);

  if(plotCongMapGP.found())
    congMap.plotCongMap(plotCongMapGP);

  if(plotCongMapXPM.found())
    congMap.plotXPMCongMap(plotCongMapXPM);

  return 0;
}


