/**************************************************************************
***    
***    Copyright (c) 1995-1998 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA, 
***    and by the Regents of the University of California.    
***    All rights reserved.   
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/
/*
   1998.3.6  Devendra Vidhani

   A violationReport indicates illegalities detected for each
   cell in an RBPlacement.
*/


#ifndef _VIOLREPORT_H_
#define _VIOLREPORT_H_

#include <vector>
#include "ABKCommon/abkcommon.h"
#include "RBPlace/RBPlacement.h"
#include "RBPlace/violations.h"


class ViolationReport 
{
    const DB& 		   _db;
    RBPlacement*           _rbplace;
    const PlacementWOrient _pl;

    std::vector <Violations>    _violations;
    unsigned               _numViols;
    Verbosity              _verbosity;


    void 	_checkLegality();
    unsigned 	_isPlacementRowBased();

    void _checkHorizontalOverlap();
    void _checkCellOrientation ();
    void _checkOverlapLayer ();
    void _checkSpecialWiring ();
    void _recheckSpecialWiring ();
    bool _checkOverlapObstacles (unsigned lowerCell, unsigned upperCell) const;
    bool _checkOverlap(unsigned cellID, const dbNet& snet) const;

    void logError (const Violations&, unsigned, const RBPSubRow&, const char*);
    
    void logError (const Violations&,unsigned,unsigned,const RBPSubRow&,
		   const char*);

    void logError (const Violations&, unsigned, const Orientation &,
                   const Symmetry&, const Orientation&, const Symmetry&,
		   const char*);
    
    void logError (const Violations&, unsigned, const Symmetry&,
                   const Orientation&, const char *);
    void logError (const Violations&, unsigned, unsigned, const char *);
    void logError (const Violations&, unsigned, const dbNet&, const char *);
    void logError (const Violations&, unsigned, const char *);
    
public:

    ViolationReport (const DB& db, Verbosity verbosity = Verbosity("1_0_0"));

    ViolationReport (const DB& db, RBPlacement &rowBasedStructure,
                     Verbosity verbosity = Verbosity("1_0_0"));

    bool checkOverlap(unsigned cellID, const dbLayerGeom& lGeom) const;
  
    const Violations & operator[](unsigned cellId) const
    { return _violations[cellId]; }

    bool hasViolations () const { return (_numViols == 0); }

    unsigned getNumViols () const { return _numViols; }

    void clearCell (unsigned i)
    {
	if(_violations[i].hasViolations())
        {
            _violations[i].clearAll();
            _numViols--;
	}
    }
};

#endif
