/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
*** Copyright (c) 2000-2002 Regents of the University of Michigan,
***               Saurabh N. Adya and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/






















//created by Saurabh Adya, Dec 3 2000
#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include <algorithm>
#include <list>
#include <ABKCommon/sgi_hash_map.h>
#include "ABKCommon/abkcommon.h"
#include "RBPlacement.h"

#ifdef _MSC_VER
#ifndef rint
#define rint(a) floor((a)+0.5)
#endif
#endif


//function to search a list
bool findInList(itRBPSubRow its, list<itRBPSubRow>& seenList)
{
  std::list<itRBPSubRow>::iterator listElem;
  bool foundInList = 0;
  for(listElem = seenList.begin();
      listElem != seenList.end();
      ++listElem)
    if(its == *listElem)
      {
	foundInList = 1;
	break;
      }
  return(foundInList);
}
//added by sadya to remove overlaps in each subrow
void RBPlacement::remOverlapsSR(void)
{
  if (_params.verb.forActions) cout << " Running overlap removal ..." << endl;
  double initHPWL=evalHPWL();
  if (_params.verb.forMajStats)
                   cout << "  HPWL before overlap remover is "<<initHPWL<<endl;
  Timer tm;

  itRBPCoreRow itc;
  itRBPSubRow its;

  unsigned initOverlaps = getNumOverlaps();

  //for each core row, for each subrow scan left to right if overlap found then take action
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	shuffleSR(its);
    }
  tm.stop();
  double HPWLafter=evalHPWL();
  double HPWLwPins=evalHPWL(true);
  unsigned finalOverlaps = getNumOverlaps();

  if (_params.verb.forMajStats) 
  {
    cout<<" Removing overlaps took " << tm << endl;
    cout<<"  Discovered " << initOverlaps << " overlaps "<<endl;
    cout<<"  Overlaps remaining: "<<finalOverlaps<<endl;
    cout<<"  HPWL after  overlap removal is "<<HPWLafter<<endl;
    cout<<"  HPWL w pins                 is "<<HPWLwPins<<endl;
    cout<<"  % change in HPWL is "<<(HPWLafter-initHPWL)*100/HPWLafter<<endl;
  }
}

//added by sadya to remove overlaps in each subrow
void RBPlacement::remOverlapsVert(void)
{
  if (_params.verb.forActions) 
    cout << " Running overlap removal with Vertical Moves..." << endl;
  double initHPWL=evalHPWL();
  if (_params.verb.forMajStats)
    cout << "  HPWL before overlap remover is "<<initHPWL<<endl;
  Timer tm;
  
  itRBPCoreRow itc;
  itRBPSubRow its;
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  itRBPCoreRow itcNext;   //used to scan immediate next row
  itRBPSubRow itsNext;    //used to scan immediate next sub row 
  itRBPCellIds itnNext;   //used to scan immediate next sub row cells
  itRBPCellIds itnright;   //used to scan immediate next sub row cells
  itRBPCellIds itnNextLeft; //used to scan 
  itRBPCellIds itnNextRight; //used to scan 
  
  RBPSubRow tempSR;
  
  bool leftRight=false;   //false means move left else move right
  double WS=0;
  unsigned numOverlaps=0;
  unsigned totalOverlaps=0;
  double rowHeight=0;
  double xDiff=0;
  //for each core row, for each subrow scan left to right if overlap found then take action
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      rowHeight=itc->getHeight();
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      //check for overlaps in the same row
	      xDiff=1;
	      if(itn!=its->cellIdsEnd()-1)
		{
		  itntemp=itn+1;
		  xDiff= _placement[*itn].x + 
		    _hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;
		  if(xDiff > 1e-10)
		    {
		      //overlap found, now deal with it
		      numOverlaps++;
		      totalOverlaps++;

		      itnright = itntemp;

		      //check for WS in upper rows
		      if(itc!=_coreRows.end()-1)
			{
			  itcNext = itc+1;
			  for(itsNext=itcNext->subRowsBegin();
			      itsNext!=itcNext->subRowsEnd();++itsNext)
			    {
			      for(itnNext=itsNext->cellIdsBegin();
				  itnNext!=itsNext->cellIdsEnd()-1;++itnNext)
				{
				  //check for WS in the upper adjacent  row
				  if((_placement[*itnNext].x <=
				      _placement[*itn].x) &&
				     (_placement[*(itnNext+1)].x >=
				      _placement[*itn].x))
				    break;   //closest cell found in upper
				             //row
				}
			      itnNextLeft = itnNext;
			      itnNextRight = itnNext;
			      
			      while(itnNextRight!=itsNext->cellIdsEnd()-1 ||
				    itnNextLeft!=itsNext->cellIdsBegin())
				{
				  if(leftRight==false && 
				     itnNextRight!=itsNext->cellIdsEnd()-1)
				    leftRight=true;
				  else if(leftRight==true && 
					  itnNextLeft!=itsNext->cellIdsBegin())
				    leftRight=false;
				  
				  if(leftRight == true)  //search right
				    {
				      itntemp=itnNextRight+1;
				      WS=_placement[*itntemp].x -
					_placement[*itnNextRight].x -
					_hgWDims->getNodeWidth(*itnNextRight);
				      if(WS > 0)   //white space found
					{
					  if(WS >=_hgWDims->getNodeWidth(*itn))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itn);
					      _placement[*itn].x = 
						_placement[*itnNextRight].x;
					      _placement[*itn].y = 
						itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itn);
					      --numOverlaps;
					      break;
					    }
					  else if
					    (WS>=
					     _hgWDims->getNodeWidth(*itnright))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itnright);
					      _placement[*itnright].x = 
						_placement[*itnNextRight].x;
					      _placement[*itnright].y = 
						itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itnright);
					      --numOverlaps;
					      break;
					    }
					  else
					    {
					      
					    }
					}
				      ++itnNextRight;
				    }
				  else //search left for WS
				    {
				      itntemp=itnNextLeft-1;
				      WS = _placement[*itnNextLeft].x -
					_placement[*itntemp].x -
					_hgWDims->getNodeWidth(*itntemp);
				      if(WS > 0)   //white space found
					{
					  if(WS >=_hgWDims->getNodeWidth(*itn))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itn);
					      _placement[*itn].x=
						_placement[*itntemp].x;
					      _placement[*itn].y=
						itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itn);
					      --numOverlaps;
					      break;
					    }
					  else if
					    (WS >= 
					     _hgWDims->getNodeWidth(*itnright))
					    {
					      tempSR = *its;
					      tempSR.removeCell(*itnright);
					      _placement[*itnright].x = 
						_placement[*itntemp].x;
					      _placement[*itnright].y=
						itsNext->getYCoord();
					      tempSR = *itsNext;
					      tempSR.addCell(*itnright);
					      --numOverlaps;
					      break;
					    }
					}
				      itnNextLeft--;
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }
  tm.stop();
  double HPWLafter=evalHPWL();
  double HPWLwPins=evalHPWL(true);
  if (_params.verb.forMajStats) 
    {
      cout<<" Removing overlaps took " << tm << endl;
      cout<<"  Discovered " << totalOverlaps << " overlaps "<<endl;
      cout<<"  Overlaps remaining: "<<numOverlaps<<endl;
      cout<<"  HPWL after  overlap removal is "<<HPWLafter<<endl;
      cout<<"  HPWL w pins                 is "<<HPWLwPins<<endl;
      cout<<"  % change in HPWL is "<<(HPWLafter-initHPWL)*100/HPWLafter<<endl;
    }
}

void RBPlacement::snapCellsInSites(void)
{
  itRBPCoreRow itc;
  itRBPSubRow its;
  itRBPCellIds itn;
  //itRBPCellIds itntemp;
  double siteSpacing;
  int index;
  
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  siteSpacing = its->getSpacing();
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      if(int(_placement[*itn].x)%int(siteSpacing) != 0 || 
		 _placement[*itn].x != int(_placement[*itn].x))
		{
		  //cout<<"cell not in site "<<endl;
		  index = *itn;
		  int newPosX = int(_placement[*itn].x) 
		    - int(_placement[*itn].x)%int(siteSpacing);

		  _placement[*itn].x = newPosX;
		}
	    }
	}
    }  
}


void RBPlacement::remOverlaps(void)
{
  if (_params.verb.forActions) cout << " Running overlap removal ..." << endl;
  double initHPWL=evalHPWL();
  if (_params.verb.forMajStats)
                   cout << "  HPWL before overlap remover is "<<initHPWL<<endl;
  Timer tm;

  itRBPCoreRow itc;
  itRBPCoreRow itcNext;   //used to scan immediate next row
  itRBPCoreRow itcCurr;   //used to scan immediate next row
  itRBPSubRow its;
  itRBPSubRow itsCurr;    //used to track current sub row in bfs
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  //itRBPCellIds itnleft;
  //itRBPCellIds itnright;
  itRBPCellIds itnToMove;
  itRBPSubRow itsNext;    //used to scan next sub row 
  //itRBPSubRow itsTemp;    //temp
  //itRBPCellIds itnNext;   //used to scan immediate next sub row cells

  //bool leftRight=false;   //false means move left else move right
  //double WS=0;
  //char plotFile[20];
  double rowHeight=0;
  double nodeHeight=0;
  double xDiff=0;
  const unsigned threshold = 50;

  unsigned initOverlaps = getNumOverlaps();
  //for each core row, for each subrow scan left to right if overlap found then take action
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      rowHeight=itc->getHeight();
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  shuffleSR(its);
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      nodeHeight=_hgWDims->getNodeHeight(*itn);
	      //check for overlaps in the same row
	      xDiff=1;
	      if(itn!=its->cellIdsEnd()-1)
		{
		  itntemp=itn+1;
		  xDiff= _placement[*itn].x + 
		    _hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;

		  if(xDiff > 1e-10)
		    {
		      //overlap found, now deal with it

		      double srOrigStart = its->getXStart();
		      double srOrigEnd = srOrigStart + its->getNumSites()*
			                               its->getSiteWidth();
		      list<itRBPSubRow> bfsList;
		      list<itRBPSubRow> seenList;
		      bfsList.push_back(its);
		      seenList.push_back(its);

		      //try to move the smallest node
		      if(_hgWDims->getNodeWidth(*itn) > 
			 _hgWDims->getNodeWidth(*(itn+1)))
			itnToMove = itn+1;
		      else
			itnToMove = itn;

		      unsigned cutoffCtr = 0;
		      while(cutoffCtr < threshold && !bfsList.empty())
			{
			  ++cutoffCtr;
			  itsCurr = *(bfsList.begin());
			  bfsList.pop_front();
			  itcCurr = static_cast<itRBPCoreRow>
				      (&itsCurr->getCoreRow());

			  double srStart = itsCurr->getXStart();
			  double srEnd = srStart + itsCurr->getNumSites()*
			                           itsCurr->getSiteWidth();

			  double srWidth = srEnd - srStart;
			  double cellWidths=0;
			  for(itntemp = itsCurr->cellIdsBegin(); 
			      itntemp != itsCurr->cellIdsEnd(); ++itntemp)
			    cellWidths += _hgWDims->getNodeWidth(*itntemp);

			  double widthToMove = 
			    _hgWDims->getNodeWidth(*itnToMove);

//cout<<" overlap x "<<_placement[*itn].x<<" YCoord: "<<itsCurr->getYCoord()<<" XCoords: "<<srStart<<"  "<<srEnd<<" list Size: "<<bfsList.size()<<" widthToMove "<<widthToMove<<" space "<<(srWidth-cellWidths)<<endl;
			  //found sr to move
			  if(widthToMove <= (srWidth-cellWidths)) 
			    {
//cout<<"found subrow "<<widthToMove<<"  "<<(srWidth-cellWidths)<<" "<<_hgWDims->getNodeByIdx(*itnToMove).getName()<<" "<<*itnToMove<<endl;
			      double xLocToMoveTo=0;

			      if(srEnd <= srOrigStart) //move left most cell
				{
				  double leftNodeWidth = 
				  _hgWDims->getNodeWidth(*its->cellIdsBegin());
				  if(leftNodeWidth <= widthToMove)
				    {
				      itnToMove = its->cellIdsBegin();
				      widthToMove = leftNodeWidth;
				    }
				  xLocToMoveTo = srEnd - widthToMove;
				}
			      else if(srStart >= srOrigEnd) //move right cell
				{
				  double rightNodeWidth = 
				  _hgWDims->getNodeWidth(*(its->cellIdsEnd()-1));
				  if(rightNodeWidth <= widthToMove)
				    {
				      itnToMove = its->cellIdsEnd()-1;
				      widthToMove = rightNodeWidth;
				    }
				  xLocToMoveTo = srStart;
				}
			      else//move the current cell
				{
				  if(srEnd <= _placement[*itnToMove].x)
				    xLocToMoveTo = srEnd - widthToMove;
				  else if(srStart >= _placement[*itnToMove].x)
				    xLocToMoveTo = srStart;
				  else
				    xLocToMoveTo = _placement[*itnToMove].x;
				}
		              unsigned indexToMove = *itnToMove;

		              RBPSubRow& tempSR1=const_cast<RBPSubRow&>(*its);
			      tempSR1.removeCell(indexToMove);
			      _placement[indexToMove].x = xLocToMoveTo;
			      _placement[indexToMove].y = itsCurr->getYCoord();
			      RBPSubRow& tempSR2 = 
				const_cast<RBPSubRow&>(*itsCurr);
			      tempSR2.addCell(indexToMove);
			      shuffleSR(itsCurr);
			      shuffleSR(its);
			      break;
			    }

			  //add new elements to bfsList
			  if(itsCurr != itcCurr->subRowsEnd()-1)
			    {
			      itsNext = itsCurr+1;
			      if(findInList(itsNext, seenList) == 0)
				{
				  bfsList.push_back(itsNext);
				  seenList.push_back(itsNext);
				}
			    }
			  if(itsCurr != itcCurr->subRowsBegin())
			    {
			      itsNext = itsCurr-1;
			      if(findInList(itsNext, seenList) == 0)
				{
				  bfsList.push_back(itsNext);
				  seenList.push_back(itsNext);
				}
			    }
			  if(itcCurr != _coreRows.end()-1)
			    {
			      itcNext = itcCurr+1;
			      for(itsNext = itcNext->subRowsBegin();
				  itsNext!=itcNext->subRowsEnd();++itsNext)
				{
				  double nextsrStart = itsNext->getXStart();
				  double nextsrEnd = nextsrStart + 
				    itsNext->getNumSites()*
				    itsNext->getSiteWidth();
				  
				  if((nextsrStart <= srStart && 
				      nextsrEnd >= srStart) || 
				     (nextsrStart >= srStart &&
				      nextsrEnd <= srEnd) ||
				     (nextsrStart <= srEnd &&
				      nextsrEnd >= srEnd)) //overlapping SR
				    {
				      if(findInList(itsNext, seenList) == 0)
					{
					  bfsList.push_back(itsNext);
					  seenList.push_back(itsNext);
					}
				    }
				}
			    }
			  if(itcCurr != _coreRows.begin())
			    {
			      itcNext = itcCurr-1;
			      for(itsNext = itcNext->subRowsBegin();
				  itsNext!=itcNext->subRowsEnd();++itsNext)
				{
				  double nextsrStart = itsNext->getXStart();
				  double nextsrEnd = nextsrStart + 
				    itsNext->getNumSites()*
				    itsNext->getSiteWidth();
				  
				  if((nextsrStart <= srStart && 
				      nextsrEnd >= srStart) || 
				     (nextsrStart >= srStart &&
				      nextsrEnd <= srEnd) ||
				     (nextsrStart <= srEnd &&
				      nextsrEnd >= srEnd)) //overlapping SR
				    {
				      if(findInList(itsNext, seenList) == 0)
					{
					  bfsList.push_back(itsNext);
					  seenList.push_back(itsNext);
					}
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }
  tm.stop();
  double HPWLafter=evalHPWL();
  double HPWLwPins=evalHPWL(true);
  unsigned finalOverlaps = getNumOverlaps();

  if (_params.verb.forMajStats) 
  {
    cout<<" Removing overlaps took " << tm << endl;
    cout<<"  Discovered " << initOverlaps << " overlaps "<<endl;
    cout<<"  Overlaps remaining: "<<finalOverlaps<<endl;
    cout<<"  HPWL after  overlap removal is "<<HPWLafter<<endl;
    cout<<"  HPWL w pins                 is "<<HPWLwPins<<endl;
    cout<<"  % change in HPWL is "<<(HPWLafter-initHPWL)*100/HPWLafter<<endl;
  }
}

void RBPlacement::shuffleSR(itRBPSubRow its)
{
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  itRBPCellIds itnleft;
  itRBPCellIds itnright;
  //itRBPSubRow itsNext;    //used to scan immediate next sub row 
  //itRBPCellIds itnNext;   //used to scan immediate next sub row cells

  bool leftRight=false;   //false means move left else move right
  double WS=0;
  //double rowHeight=0;
  double nodeHeight=0;
  double xDiff=0;  

  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
    {
      nodeHeight=_hgWDims->getNodeHeight(*itn);
      //check for overlaps in the same row
      xDiff=1;
      if(itn!=its->cellIdsEnd()-1)
	{
	  itntemp=itn+1;
	  xDiff= _placement[*itn].x + 
	    _hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;
	  
	  if(xDiff > 1e-10)
	    {
	      //overlap found, now deal with it
	      
	      itnleft=itn;
	      itnright=itn;
	      leftRight=false;
	      
	      while(xDiff>1e-10 && (itnleft!=(its->cellIdsBegin()) || 
				    itnright!=(its->cellIdsEnd()-2)))
		{
		  // chose the direction of search
		  if(leftRight==false && itnright!=its->cellIdsEnd()-2)
		    leftRight=true;
		  else if(leftRight==true && 
			  itnleft!=its->cellIdsBegin())
		    leftRight=false;
		  
		  if(leftRight==true)//search right for WS
		    {
		      ++itnright;
		      itntemp=itnright+1;
		      WS=_placement[*itntemp].x -
			_placement[*itnright].x -
			_hgWDims->getNodeWidth(*itnright);
		      if(WS > 0) //WS found
			{
			  if(WS > xDiff)
			    WS=xDiff;
			  xDiff-=WS;
			  for(itntemp=itn+1;itntemp!=itnright+1;
			      ++itntemp)
			    _placement[*itntemp].x+=WS;
			}
		    }
		  else //search left for WS
		    {
		      itntemp=itnleft;
		      --itnleft;
		      WS=_placement[*itntemp].x -
			_placement[*itnleft].x -
			_hgWDims->getNodeWidth(*itnleft);
		      if(WS > 0) //WS found
			{
			  if(WS > xDiff)
			    WS=xDiff;
			  xDiff-=WS;
			  for(itntemp=itn;itntemp!=itnleft;--itntemp)
			    {
			      _placement[*itntemp].x-=WS;
			    }
			}
		    }
		}
	    }
	}
    }
}

unsigned RBPlacement::getNumOverlaps()
{
  unsigned overlaps = 0;
  itRBPCellIds itn;
  itRBPCellIds itntemp;
  itRBPCoreRow itc;
  itRBPSubRow its;

  double xDiff=0;
  for(itc=_coreRows.begin();itc!=_coreRows.end();++itc)
    {
      for(its=itc->subRowsBegin();its!=itc->subRowsEnd();++its)
	{
	  for(itn=its->cellIdsBegin();itn!=its->cellIdsEnd();++itn)
	    {
	      //check for overlaps in the same row
	      xDiff=1;
	      if(itn!=its->cellIdsEnd()-1)
		{
		  itntemp=itn+1;
		  xDiff= _placement[*itn].x + 
		    _hgWDims->getNodeWidth(*itn)-_placement[*itntemp].x;
		  
		  if(xDiff > 1e-10)
		    overlaps++;
		}
	    }
	}
    }
  return(overlaps);
}

